data_lake = {
    "name": "DataLake",
    "type": "Microsoft.DataFactory/factories/linkedservices",
    "properties": {
        "annotations": [],
        "type": "AzureBlobFS",
        "typeProperties": {
            "url": "https://exampledatalakename.dfs.core.windows.net"
        }
    }
}
