schedule = {
    "name": "schedule",
    "properties": {
        "annotations": [],
        "runtimeState": "Started",
        "pipelines": [],
        "type": "ScheduleTrigger",
        "typeProperties": {
            "recurrence": {}
        }
    }
}
